#include <allegro.h>
#include "fmod/fmod.h"
#include "fmod/fmod_errors.h"
#include "sound.h"

FMUSIC_MODULE *bgmusic;

//0 on success.
int InitSound(void)
{
    bgmusic = NULL;
    
    if (FSOUND_GetVersion() < FMOD_VERSION)
    {
        allegro_message("Wrong FMOD library detected. You should be using FMOD %.02f\n", FMOD_VERSION);
        return 1;
    }
    
    /*
        INITIALIZE
    */
    if (!FSOUND_Init(32000, 64, 0))
    {
        allegro_message("%s\n", FMOD_ErrorString(FSOUND_GetError()));
        return 1;
    }
    
    return 0;
}

void DestroySound(void)
{
     DestroySong();
     FSOUND_Close();
}

void LoadSong(char *filename)
{
     bgmusic = FMUSIC_LoadSong(filename);
}

void PlaySong(void)
{
     if(bgmusic)
         FMUSIC_PlaySong(bgmusic);
}

void StopSong(void)
{
     if(bgmusic)
     {
      FMUSIC_StopSong(bgmusic);
     }
}

void DestroySong(void)
{
     if(bgmusic)
     {
                FMUSIC_FreeSong(bgmusic);
                bgmusic = NULL;
     }
}

SOUNDEFFECT::SOUNDEFFECT()
{
    sample = NULL;
}

SOUNDEFFECT::~SOUNDEFFECT()
{
    DestroySound();    
    soundOverlapping = true;                       
}

void SOUNDEFFECT::LoadSound(char *filename)
{
     sample = FSOUND_Sample_Load(FSOUND_UNMANAGED, filename, FSOUND_NORMAL | FSOUND_2D | FSOUND_LOOP_OFF, 0, 0); 
}

void SOUNDEFFECT::PlaySound(void)
{
     if(sample)
     {
         if(soundOverlapping)
             channellHandle = FSOUND_PlaySound(FSOUND_FREE, sample);
         else 
         {
              StopSound();
              channellHandle = FSOUND_PlaySound(FSOUND_FREE, sample);
         }
     }
}

void SOUNDEFFECT::StopSound(void)
{
     if(sample && FSOUND_IsPlaying(channellHandle))
     {
               FSOUND_StopSound(channellHandle);
     }
}

void SOUNDEFFECT::DestroySound(void)
{
     if(sample)
     {
      FSOUND_Sample_Free(sample);
      sample = NULL;
     }
}

void SOUNDEFFECT::SetRepeat(bool onoff)
{
     if(onoff)
     FSOUND_Sample_SetMode(sample, FSOUND_LOOP_NORMAL);
     else
     FSOUND_Sample_SetMode(sample, FSOUND_LOOP_OFF);
}

void SOUNDEFFECT::AllowSoundOverlapping(bool allow)
{
     soundOverlapping = allow;
}
