#include <allegro.h>
#include <math.h>
#include "graphics.h"
#include "sprite.h"
#include "input.h"
#include "timer.h"
#include "trollrun.h"
#include "vector.h"
#include "stopdisk.h"
#include "killmario.h"
#include "unlocks.h"

TROLLRUN::TROLL::TROLL()
{    
    
}

TROLLRUN::TROLL::~TROLL()
{
    runA.DestroyImage();
    runB.DestroyImage();
    attackA.DestroyImage();
    attackB.DestroyImage();
}

void TROLLRUN::TROLL::Animate(bool animate)
{
     if(animate)
     {
         animRun.AddToProcessingList();
     }
     else
     {
         animRun.RemoveFromProcessingList();
     }
     
}

void TROLLRUN::TROLL::LoadArt(void)
{
     runA.LoadImage("troll1.bmp");
     runB.LoadImage("troll2.bmp");
     attackA.LoadImage("troll3.bmp");
     attackB.LoadImage("troll4.bmp");
     
     //Depending if animRun is 0 or 1, we display frame A or B of whatever we are doing.
    animRun.SetMin(-300);
    animRun.SetMax(2);
    animRun.SetUPS(10);
    animRun.EnableResettingAtBoundry(true);
    animRun.SetResetValue(0);
    animRun.AddToProcessingList();
    
    attacking = false;
}

void TROLLRUN::TROLL::SetAttack(void)
{
     attacking = true;
}

void TROLLRUN::TROLL::SetUnattack(void)
{
    attacking = false;
}
           
void TROLLRUN::TROLL::Display(int screenX, int screenY, double angle)
{    
     if(attacking)
     {
         if(animRun.GetValueFloor() < 1)
         {
              attackA.DisplayRotatedXY(screenX, screenY, angle);
         }
         else
         {
             attackB.DisplayRotatedXY(screenX, screenY, angle);
         }
     }
     else
     {
         if(animRun.GetValueFloor() < 1)
         {
              runA.DisplayRotatedXY(screenX, screenY, angle);
         }
         else
         {
             runB.DisplayRotatedXY(screenX, screenY, angle);
         }
     }
}

void TROLLRUN::EXPLODERLIST::AddExploder(int startingXoffset, int startingY)
{
     EXPLODER *newExploder;
     int counter = 0;
     
     newExploder = new EXPLODER; //Hee hee.  =)
     
     newExploder->particle = create_bitmap(2, 2);
     clear_to_color(newExploder->particle, makecol(128, 0, 0));
     newExploder->transparancy.SetValue(255);
     newExploder->transparancy.SetUPS(-128);
     newExploder->transparancy.AddToProcessingList();
     
     while(counter < exploderParticles)
     {
         newExploder->x[counter].SetValue(startingXoffset);
         newExploder->x[counter].SetUPS((rand() % 200) + 550);
         newExploder->x[counter].AddToProcessingList();
         newExploder->y[counter].SetValue(startingY);
         newExploder->y[counter].SetUPS(-1 *((rand() % 200) + 200));
         newExploder->y[counter].AddToProcessingList();
         counter++;
     }
     
     if(head == NULL)
     {
         head = newExploder;
     }
     else
     {
         EXPLODER *runner;
         runner = head;
         while(runner->next != NULL)
         {
             runner = runner->next;
         }
         runner->next = newExploder;
     }
}

void TROLLRUN::EXPLODERLIST::DrawAndUpdateAll(int xOffset)
{
     EXPLODER *runner;
     EXPLODER *deleteme;
     
     runner = head;
     
     //check to see if head is done.
     if(head && head->transparancy.GetValueFloor() <= 0)
     {
         deleteme = head;
         head = head->next;
         delete deleteme;
         runner = head;
     }
     while(runner != NULL)
     {
         if(runner->next)
         {
            if(runner->next->transparancy.GetValueFloor() <= 0)
            {
               deleteme = runner->next;
               runner->next = runner->next->next;
               delete deleteme;
            }
         }
         
         set_trans_blender(255,255,255,runner->transparancy.GetValueFloor());
         for(int counter = 0; counter < exploderParticles; counter++)
         {
             draw_trans_sprite(back_buffer, runner->particle, runner->x[counter].GetValueFloor() - xOffset, runner->y[counter].GetValueFloor()); 
             runner->yUPSDecrease = runner->yUPSDecrease + (0.25 * FrameTime());
             runner->y[counter].SetUPS(runner->y[counter].GetUPS() + runner->yUPSDecrease);
         }
         
         runner = runner->next;
     }     
}

void TROLLRUN::EXPLODERLIST::ClearAll(void)
{
     if(head)     
     {
         head->DeleteAllSubNodes();
         delete head;
         head = NULL;
     }
}

TROLLRUN::TROLLRUN()
{
                    sinehills = NULL;
                    sky = NULL;
}

void TROLLRUN::InitMisc(void)
{
                    distanceX.SetValue(-700);  //Troll doesn't begin yet.
                    distanceX.SetUPS(200); //Run at X pixels per second.
                    distanceX.AddToProcessingList();
                    
                    cloudAX.SetValue(600);
                    cloudAX.SetUPS(-80);
                    cloudAX.AddToProcessingList();
                    
                    cloudBX.SetValue(400);
                    cloudBX.SetUPS(-80);
                    cloudBX.AddToProcessingList();
                    
                    cloudCX.SetValue(200);
                    cloudCX.SetUPS(-80);
                    cloudCX.AddToProcessingList();
                    
                    cloudDX.SetValue(0);
                    cloudDX.SetUPS(-80);
                    cloudDX.AddToProcessingList();
                    
                    cloudAY = GenerateCloudY();
                    cloudBY = GenerateCloudY();
                    cloudCY = GenerateCloudY();
                    cloudDY = GenerateCloudY();
                    
                    combo = 0;
                    misses = 0;
                    hits = 0;
                    bads = 0;
                    health = 200;
                    
                    comboTrans.SetValue(255);
                    comboTrans.SetUPS(-750);
                    comboTrans.AddToProcessingList();
}

void TROLLRUN::DrawSkyInMemory(int redtop, int greentop, int bluetop, int redbottom, int greenbottom, int bluebottom)
{
     int y = 0;
     int reddiff = redtop - redbottom;
     int greendiff = greentop - greenbottom;
     int bluediff = bluetop - bluebottom;
     
     sky = create_bitmap(SCREEN_W, SCREEN_H);
     
     while(y < SCREEN_H)
     {
             double ypercent = (double)y / (double)SCREEN_H;
             hline(sky, 0, y, SCREEN_W, makecol(redtop - (int)(ypercent*reddiff), greentop - (int)(ypercent*reddiff), bluetop - (int)(ypercent*bluediff)));
             y++;
     }
}

void TROLLRUN::DrawSky(void)
{
     blit(sky, back_buffer, 0, 0, 0, 0, SCREEN_W, SCREEN_H);
}

void TROLLRUN::DrawSineHillsInMemory(void)
{
     int x;
     int y;
     int counter;
     
     sinehills = create_bitmap(1280, 480);
     
     clear_to_color(sinehills, makecol(255, 0, 255));
     
     for(x = 0; x < (SCREEN_W * 2); x++)
     {
           counter = 0;
           y = CalculateYFromSineHill(x);
           while(y < 480)
           {
            if(counter < 2)
            {
                 putpixel(sinehills,x, y, makecol(0, 128, 0));
            }
            else
            {
                putpixel(sinehills,x, y, makecol(0, 200-(y/10), 0));
            }
            counter++;
            y++;
           }
     }
}

void TROLLRUN::LoadSprites(void)
{
     hillbg.LoadImage("hillbg.bmp");
     cloud.LoadImage("cloud.bmp");
     peasantA.LoadImage("peasant1.bmp");
     peasantB.LoadImage("peasant2.bmp");
     peasantC.LoadImage("peasant3.bmp");
     healhBar.LoadImage("bar.bmp");
     barRed.LoadImage("barred.bmp");
     barGreen.LoadImage("bargreen.bmp");
     miniGameFlag.LoadImage("mgflag.bmp");
     comboFont = alfont_load_font("carbon.ttf");
     alfont_set_font_size(comboFont, 24);
     percentFont = alfont_load_font("carbon.ttf");
     alfont_set_font_size(percentFont, 24);
     comboFontLarge = alfont_load_font("carbon.ttf");
     alfont_set_font_size(comboFontLarge,36);
}

void TROLLRUN::DestroySprites(void)
{
     hillbg.DestroyImage();
     cloud.DestroyImage();
     peasantA.DestroyImage();
     peasantB.DestroyImage();
     peasantC.DestroyImage();
     healhBar.DestroyImage();
     barRed.DestroyImage();
     barGreen.DestroyImage();
     destroy_bitmap(sinehills);
     destroy_bitmap(sky);
     miniGameFlag.DestroyImage();
     alfont_destroy_font(comboFont);
     alfont_destroy_font(comboFontLarge);
     alfont_destroy_font(percentFont);
     hitMissBanner.DestroyFont();
}

void TROLLRUN::InitPeasants(void)
{
     int counter = 0;
     
     while(counter < numPeasants)
     {
         peasants[counter].LinkToSpriteArt(&peasantA, &peasantB, &peasantC);
         SetPeasantLoc(counter, 700 + rand() % 700, -100);
         counter++;
     }
}

void TROLLRUN::DrawAllPeasants(void)
{
     int counter = 0;
     int relativeX = distanceX.GetValueFloor();
     if(relativeX < 0)
     {
          relativeX = 0;
     }
     
     while(counter < numPeasants)
     {
         if(playing && !peasants[counter].GetEscaped() && (peasants[counter].GetXLoc() - 220) < relativeX)
         {
              peasants[counter].SetEscaped();                             
              hitMissBanner.NewText(1);     
              combo = 0;
              misses++; 
              health = health - 30;                     
              if(health < 30)
                  health = 0;
         }
         peasants[counter].Draw(peasants[counter].GetXLoc() - relativeX, CalculateYFromSineHill(peasants[counter].GetXLoc() + 30) - 78, CalculateTangentAngleForX(peasants[counter].GetXLoc() + 30));
         if((peasants[counter].GetXLoc() + 70) < relativeX)
         {
             peasants[counter].DestroyPeasant();
             SetPeasantLoc(counter, relativeX + 800 + rand() % 800, -100);                           
         }
         counter++;
     }
}

void TROLLRUN::DrawHillBG()
{
     int x = (-1 * distanceX.GetValueFloor()) / 2;
     int ypos;
     
     if(x > 0)
     {    
          x = 0;
     }
      while(x > 0)
      {
       x = x - hillbg.GetWidth();
      }
 
      ypos = SCREEN_H - hillbg.GetHeight();
 
      while(x < SCREEN_W)
      {
         hillbg.DisplayMaskedXY(x, ypos);
         x = x + hillbg.GetWidth();
      }
     
}

int TROLLRUN::GenerateCloudY(void)
{
     return rand() % 200;
}

void TROLLRUN::DrawClouds(void)
{
     if(cloudAX.GetValueFloor() <= -200)
     {
         cloudAX.SetValue(640);
         cloudAY = GenerateCloudY();
     }
     
     if(cloudBX.GetValueFloor() <= -200)
     {
         cloudBX.SetValue(640);
         cloudBY = GenerateCloudY();
     }
     
     if(cloudCX.GetValueFloor() <= -200)
     {
         cloudCX.SetValue(640);
         cloudCY = GenerateCloudY();
     }
     
     if(cloudDX.GetValueFloor() <= -200)
     {
         cloudDX.SetValue(640);
         cloudDY = GenerateCloudY();
     }
     cloud.DisplayMaskedXY(cloudAX.GetValueFloor(), cloudAY);
     cloud.DisplayMaskedXY(cloudBX.GetValueFloor(), cloudBY);
     cloud.DisplayMaskedXY(cloudCX.GetValueFloor(), cloudCY);
     cloud.DisplayMaskedXY(cloudDX.GetValueFloor(), cloudDY);
}

void TROLLRUN::SetPeasantLoc(int peasantNumber, int Loction, double speed)
{
     peasants[peasantNumber].SpawnPeasant(Loction, speed, true);
}

void TROLLRUN::CollisionDetectPeasants(void)
{
     int counter = 0;
     bool hitone = false;;
     
     int screenloc;
     
     while(counter < numPeasants)
     {
         screenloc = peasants[counter].GetXLoc() - distanceX.GetValueFloor();
         if(peasants[counter].IsAlive() && screenloc < 350 && screenloc > 240)
         {
             comboTrans.SetValue(255);                           
             combo++;
             hits++;
             health++;
             health+=(combo / 25);
             if(health > 300)
                 health = 300;
             score+= 50 + (2 * combo);
             hitMissBanner.NewText(0);
             hitone = true;
             peasantSmush.PlaySound();
             exploders.AddExploder(peasants[counter].GetXLoc(), CalculateYFromSineHill(peasants[counter].GetXLoc() + 30));
             peasants[counter].DestroyPeasant();
         }
         counter++;
     }
     if(hitone == false)
     {
       combo = 0;
       health = health - 20;
       if(health < 0)
           health = 0;
       bads++;
       hitMissBanner.NewText(2);
     }
}

void TROLLRUN::PauseAction(void)
{
     distanceX.RemoveFromProcessingList();
     cloudAX.RemoveFromProcessingList();
     cloudBX.RemoveFromProcessingList();
     cloudCX.RemoveFromProcessingList();
     cloudDX.RemoveFromProcessingList();
     troll.Animate(false);
     
     int counter = 0;
     while(counter < numPeasants)
     {
         peasants[counter].Animate(false);
         counter++;
     }
}

void TROLLRUN::ResumeAction(void)
{
     distanceX.AddToProcessingList();
     cloudAX.AddToProcessingList();
     cloudBX.AddToProcessingList();
     cloudCX.AddToProcessingList();
     cloudDX.AddToProcessingList();
     troll.Animate(true);
     
     int counter = 0;
     while(counter < numPeasants)
     {
         peasants[counter].Animate(true);
         counter++;
     }
}

void TROLLRUN::DrawComboText(void)
{
     int yloc = 180;
     
     if(combo > 1 && comboTrans.GetValueFloor() > 0)
     {
              BITMAP *temp;
              if(combo >= 100)
              {
                  temp = create_bitmap(640, 36);
                  clear_to_color(temp, makecol(255, 0, 255));
                  alfont_textprintf_centre_ex(temp,  comboFontLarge, 320, 0, makecol(240, 70, 70), makecol(255, 0, 255), "COMBO: %d", combo);
                  set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
                  draw_trans_sprite(back_buffer, temp, 0, yloc);
                  destroy_bitmap(temp);
              }
              if(combo < 100)
              {
                  temp = create_bitmap(640, 24);
                  clear_to_color(temp, makecol(255, 0, 255));
                  yloc = yloc + 6;
                  alfont_textprintf_centre_ex(temp,  comboFont, 320, 0, makecol(240, 70, 70), makecol(255, 0, 255), "COMBO: %d", combo);
                  set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
                  draw_trans_sprite(back_buffer, temp, 0, yloc);
                  destroy_bitmap(temp);
              }
     }
}

//Notes about combo bar.
//Bar BMP is 160 pixels wide.
//Individuals bars are + 6 pixels down from the top.
//First bar is pixel 6.
//All bars are +5 away from each other.  6 11 16 21
//There are 30 bars total.
//Bars 0-20 are red.  21-29 are green.
//Heath = 0-300
void TROLLRUN::DrawHealthBar(void)
{
     const int x = 420;
     const int y = 440;
     int barCounter = 0;
     int counter = 0;
     healhBar.DisplayXY(x, y);
     
     while(barCounter < health)
     {
         if(counter < 21)
         {
             barRed.DisplayXY(x + 6 + (counter * 5), y + 5);      
         }
         else if (counter < 30)
         {
             barGreen.DisplayXY(x + 6 + (counter * 5), y + 5);
         }
         barCounter = barCounter + 10;
         counter++;
     }              
     BITMAP *tempy;
     tempy = create_bitmap(640, 24);
     clear_to_color(tempy, makecol(255, 0, 255));
     alfont_textprintf_ex(tempy,  percentFont, x + 165, 0, makecol(0, 100, 0), makecol(255, 0, 255), "%d%%", (health / 3));
     set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
     masked_blit(tempy, back_buffer, 0, 0, 0, y+2, 640, 24);
     destroy_bitmap(tempy);
}

void TROLLRUN::DrawSineHill(unsigned int xoffset)
{
     int distance;
     distance = distanceX.GetValueFloor();
     
     if(distance < 0)
     masked_blit(sinehills, back_buffer, xoffset, 0, 0, 0, SCREEN_W, SCREEN_H);
     else
     {
         while(distance >= 1280)
         {
             distance = distance - 1280;
         }
         masked_blit(sinehills, back_buffer, distance, 0, 0, 0, SCREEN_W, SCREEN_H);
         if(distance > 640)
         {
             masked_blit(sinehills, back_buffer, 0, 0, (1280 - distance), 0, SCREEN_W, SCREEN_H);        
         }
     }
}

//There will be a minigame every other crest.
//First crest is at 640.
//Each crest is 1280 away.
void TROLLRUN::DisplayMiniGameFlags(void)
{
    //counter goes through every possible location.                            
    int counter = 639 + 1279;
    
    while(counter < distanceX.GetValueFloor() + 700)
    {
         miniGameFlag.DisplayMaskedXY(counter - distanceX.GetValueFloor() - 60, CalculateYFromSineHill(640) - miniGameFlag.GetHeight() + 1);        
         counter += 2560;
    }
}

void TROLLRUN::DisplayScore(void)
{
     BITMAP *tempy;
     tempy = create_bitmap(640, 24);
     clear_to_color(tempy, makecol(255, 0, 255));
     alfont_textprintf_ex(tempy,  percentFont, 10, 0, makecol(0, 0, 0), makecol(255, 0, 255), "SCORE: %d", score);
     set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
     masked_blit(tempy, back_buffer, 0, 0, 0, 10, 640, 24);
     destroy_bitmap(tempy);
}

int TROLLRUN::CalculateYFromSineHill(int x)
{
    double anglecalc;
    int sinecalc;
    int screenpos;
    
    //When x = 0, the input should be 0 - .5PIT  x=1280 input is 2PI - .5 * PI.
    anglecalc = ((double)x * ((2.0 * M_PI)/((double)SCREEN_W * 2.0)) - (0.5 * M_PI));
    
    sinecalc = (int)(60*sin(anglecalc));
    
    //because screen coordinates start at 0,0, and get larger as the value decreases,
    //we want to add to Y to make it lower, not subtract.
    sinecalc = ((-1) * sinecalc);
    
    //Return the sine plus the screen offset (sinecalc goes from -200 -> 200, starting at 200.
    screenpos = sinecalc + 400;
    
    //outputs range from 460 to 60
    return screenpos;
}

double TROLLRUN::CalculateTangentAngleForX(int x)
{
       double angle = ((double)x * ((2.0 * M_PI)/((double)SCREEN_W * 2.0)));
       
       return (-1)*sin(angle)*9; //Last number is max angle.
}

void TROLLRUN::ScrollMiniGameAcrossScreen(void)
{
     VECTOR waittime;
     waittime.SetValue(640);
     waittime.SetUPS(-640);
     waittime.AddToProcessingList();
     
     BITMAP *tempy;
     tempy = create_bitmap(640, 36);
     clear_to_color(tempy, makecol(255, 0, 255));
     alfont_textprintf_ex(tempy,  comboFontLarge, 0, 0, makecol(255, 0, 0), makecol(255, 0, 255), "MINIGAME!");
     
     while(waittime.GetValueFloor() > -640)
     {
      DrawSky();
      DrawHillBG();
      DrawSineHill(0);
      DrawClouds();
          
      DisplayMiniGameFlags();
      DrawAllPeasants();    
          
      if(distanceX.GetValueFloor() < 0)
      {
            troll.Display(220 + distanceX.GetValueFloor(), CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
      }
      else
          troll.Display(220, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
          
      hitMissBanner.DrawBannerCenter(330, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 70);   
          
      exploders.DrawAndUpdateAll(distanceX.GetValueFloor());
      DrawComboText();
      DrawHealthBar();
      DisplayScore();
      
      int x = waittime.GetValueFloor();
      if(x < 280)
      {
           x = 280;
      }
      
      masked_blit(tempy, back_buffer, 0, 0, x, 222, 640, 36);
      
      FlipScreen();
     }
     destroy_bitmap(tempy);
     waittime.RemoveFromProcessingList();
}

void TROLLRUN::ShowWinnerLoser(bool result)
{
      VECTOR waittime;
     waittime.SetValue(3.99);
     waittime.SetUPS(-3);
     waittime.AddToProcessingList();
     
     BITMAP *tempy;
     tempy = create_bitmap(640, 36);
     clear_to_color(tempy, makecol(255, 0, 255));
     if(result)
         alfont_textprintf_ex(tempy,  comboFontLarge, 0, 0, makecol(255, 0, 0), makecol(255, 0, 255), "WINNER!");
     else
         alfont_textprintf_ex(tempy,  comboFontLarge, 0, 0, makecol(0, 0, 100), makecol(255, 0, 255), "NO GOOD");
     set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
     
     while(waittime.GetValueFloor() > 0)
     {
      DrawSky();
      DrawHillBG();
      DrawSineHill(0);
      DrawClouds();
          
      DisplayMiniGameFlags();
      DrawAllPeasants();    
          
      if(distanceX.GetValueFloor() < 0)
      {
            troll.Display(220 + distanceX.GetValueFloor(), CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
      }
      else
          troll.Display(220, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
          
      hitMissBanner.DrawBannerCenter(330, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 70);   
          
      exploders.DrawAndUpdateAll(distanceX.GetValueFloor());
      DrawComboText();
      DrawHealthBar();
      DisplayScore();
      
      masked_blit(tempy, back_buffer, 0, 0, 280, 222, 640, 36);
      
     BITMAP *temp;
     temp = create_bitmap(640, 36);
     clear_to_color(temp, makecol(255, 0, 255));
     alfont_textprintf_centre_ex(temp,  comboFontLarge, 320, 0, makecol(0, 0, 0), makecol(255, 0, 255), "%d", waittime.GetValueFloor());
     masked_blit(temp, back_buffer, 0, 0, 0, 250, 640, 36);
     destroy_bitmap(temp);
      
      FlipScreen();
     }
     destroy_bitmap(tempy);
     waittime.RemoveFromProcessingList();    
}

void TROLLRUN::DisplayGameOver(void)
{
     VECTOR waittime;
     waittime.SetValue(0);
     waittime.SetUPS(128);
     waittime.AddToProcessingList();
     
     BITMAP *tempy;
     tempy = create_bitmap(640, 36);
     clear_to_color(tempy, makecol(255, 0, 255));
     alfont_textprintf_centre_ex(tempy,  comboFontLarge, 320, 0, makecol(255, 0, 0), makecol(255, 0, 255), "GAME OVER!");
     set_trans_blender(255, 255, 255, comboTrans.GetValueFloor());
     
     BITMAP *black;
     black = create_bitmap(640, 480);
     clear_to_color(black, makecol(0, 0, 0));
     
     while(waittime.GetValueFloor() < 255)
     {
      DrawSky();
      DrawHillBG();
      DrawSineHill(0);
      DrawClouds();
          
      DisplayMiniGameFlags();
      DrawAllPeasants();    
          
      if(distanceX.GetValueFloor() < 0)
      {
            troll.Display(220 + distanceX.GetValueFloor(), CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
      }
      else
          troll.Display(220, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
          
      hitMissBanner.DrawBannerCenter(330, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 70);   
          
      exploders.DrawAndUpdateAll(distanceX.GetValueFloor());
      DrawComboText();
      DrawHealthBar();
      DisplayScore();
      
      
      set_trans_blender(255, 255, 255, waittime.GetValueFloor());
      draw_trans_sprite(back_buffer, black, 0, 0);
      
      masked_blit(tempy, back_buffer, 0, 0, 0, 222, 640, 36);
      
      FlipScreen();
      }
      
      destroy_bitmap(tempy);
      destroy_bitmap(black);
      waittime.RemoveFromProcessingList();
}

void TROLLRUN::DisplayResults(void)
{
     BITMAP *line;
     line = create_bitmap(640, 36);
     while(!ButtonADown() && !ButtonEscapeDown())
     {
            ClearScreen(0,0,0);
            
            set_trans_blender(255, 255, 255, 255);
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "RESULTS");
            draw_trans_sprite(back_buffer, line, 0, 10);     
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "HITS: %d", hits);
            draw_trans_sprite(back_buffer, line, 0, 160);   
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "MISSES: %d", misses);
            draw_trans_sprite(back_buffer, line, 0, 200);  
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "BADS: %d", bads);
            draw_trans_sprite(back_buffer, line, 0, 240);  
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "SCORE: %d", score);
            draw_trans_sprite(back_buffer, line, 0, 280);  
            
            if(score >= scoreToUnlockMario && GetMarioUnlocked() == false)
            {
                clear_to_color(line, makecol(0, 0, 0));
                alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(190, 0, 0), makecol(0, 0, 0), "MARIO ATTACK UNLOCKED!");
                draw_trans_sprite(back_buffer, line, 0, 400);  
            }
            
            if(score >= scoreToUnlockDisk && GetSpinUnlocked() == false)
            {
                clear_to_color(line, makecol(0, 0, 0));
                alfont_textprintf_centre_aa_ex(line,  comboFontLarge, 320, 0, makecol(190, 0, 0), makecol(0, 0, 0), "SPIN ATTACK UNLOCKED!");
                draw_trans_sprite(back_buffer, line, 0, 440);  
            }
                         
            FlipScreen();
     }
     
     UpdateInput();
     destroy_bitmap(line);
}

void TROLLRUN::DisplayIntro(void)
{
     BITMAP *line;
     line = create_bitmap(640, 36);
     while(!ButtonADown())
     {
            ClearScreen(0,0,0);
            
            set_trans_blender(255, 255, 255, 255);
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "THIS IS TROLLMANIA!");
            draw_trans_sprite(back_buffer, line, 0, 10); 
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "SMASH AS MANY PEASANTS AS YOU CAN!");
            draw_trans_sprite(back_buffer, line, 0, 120); 
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "USE SPACEBAR, CTRL, SHIFT, OR ENTER.");
            draw_trans_sprite(back_buffer, line, 0, 160); 
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "DON'T MISS AND DON'T LET ANY PEASANTS PASS YOU!");
            draw_trans_sprite(back_buffer, line, 0, 200); 
            
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "YOU WILL ALSO RUN INTO MINIGAMES!");
            draw_trans_sprite(back_buffer, line, 0, 240); 
                         
            clear_to_color(line, makecol(0, 0, 0));
            alfont_textprintf_centre_ex(line,  comboFontLarge, 320, 0, makecol(255, 255, 255), makecol(0, 0, 0), "HOW FAR WILL YOU GET?");
            draw_trans_sprite(back_buffer, line, 0, 420);              
                         
            FlipScreen();
     }
     destroy_bitmap(line);
}

void TROLLRUN::DoTrollRun(void)
{
     //Loopage
     VECTOR test;
     KILLMARIO killmariogame;
     STOPDISK stopdiskgame;
     LoadSprites();
     
     scoreToUnlockMario = 15000;
     //scoreToUnlockMario = 15;
     scoreToUnlockDisk = 21000;
     //scoreToUnlockDisk = 15; 
     playing = true;
     DisplayIntro();
     UpdateInput();  
     
     int minigameDifficulty = 1;
     InitMisc();
     DrawSkyInMemory(192, 229, 229, 180, 205, 222);
     DrawSineHillsInMemory();
     peasantSmush.LoadSound("smush.mp3");
     peasantSmush.AllowSoundOverlapping(false);
     hitMissBanner.LoadFont();
     troll.LoadArt();
     nextMiniGamePoint = 640 + 1280;
     
     InitPeasants();
     
     score = 0;
     
     while(!ButtonEscapeUp() && health > 0)
     {
      DrawSky();
      DrawHillBG();
      DrawSineHill(0);
      DrawClouds();
      
      if(ButtonADown())
      {
          troll.SetAttack();
          CollisionDetectPeasants();
      }
      if(ButtonAUp())
      {
          //CollisionDetectPeasants();           
          troll.SetUnattack();
      }
          
      DisplayMiniGameFlags();
      DrawAllPeasants();    
          
      if(distanceX.GetValueFloor() < 0)
      {
            troll.Display(220 + distanceX.GetValueFloor(), CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
      }
      else
          troll.Display(220, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 76, CalculateTangentAngleForX(distanceX.GetValueFloor() + 275));
          
      if(distanceX.GetValueFloor() + 280 >= nextMiniGamePoint)
      {
          PauseAction(); 
          troll.SetUnattack();  
          ScrollMiniGameAcrossScreen();
          bool win;     
          switch (rand() % 2)
          {
          case 0:
              killmariogame.LoadArt();
              win = killmariogame.PlayGame(minigameDifficulty);
              killmariogame.ReleaseArt();
              break;
          case 1:             
              stopdiskgame.LoadArt();
              win = stopdiskgame.PlayGame(minigameDifficulty);
              stopdiskgame.ReleaseArt(); 
              break;
          }
          double oldUPS = distanceX.GetUPS();
          oldUPS = oldUPS + 50;
          if(oldUPS > 550)
          {
                    oldUPS = 550;
          }
          distanceX.SetUPS(oldUPS);
          UpdateInput();
          if(win)
          {
                 score+=500;
                 health = health + 10;
          }
          else
          {
              health = health - 40;
              if(health < 0)
                 health - 0;
          }
          minigameDifficulty++;
          ShowWinnerLoser(win);
          nextMiniGamePoint+=2560;
          if(health > 0)
          ResumeAction();
      }
          
      hitMissBanner.DrawBannerCenter(330, CalculateYFromSineHill(275 + distanceX.GetValueFloor()) - 70);   
          
      exploders.DrawAndUpdateAll(distanceX.GetValueFloor());
      DrawComboText();
      DrawHealthBar();
      DisplayScore();
      FlipScreen();
     }
     
     playing = false;
     
     DisplayGameOver();
     DisplayResults();
     UpdateInput();  
     DestroySprites();
     
     if(score >= scoreToUnlockMario)
     {
              ChangeUnlock(true, GetSpinUnlocked());
              WriteUnlocks("save.dat");
     }
     
     if(score >= scoreToUnlockDisk)
     {
              ChangeUnlock(true, true);
              WriteUnlocks("save.dat");
     }
}
